@@ -409,8 +409,7 @@ sysprof_elf_get_symbol_at_address_internal (SysprofElf *self,
                                             guint64     address,
                                             guint64    *begin_address,
                                             guint64    *end_address,
-                                            guint64     text_offset,
-                                            gboolean   *is_fallback)
+                                            guint64     text_offset)
 {
   const ElfSym *symbol;
   char *ret = NULL;
@@ -421,7 +420,7 @@ sysprof_elf_get_symbol_at_address_internal (SysprofElf *self,
 
   if (self->debug_link_elf != NULL)
     {
-      ret = sysprof_elf_get_symbol_at_address_internal (self->debug_link_elf, filename, address, begin_address, end_address, text_offset, is_fallback);
+      ret = sysprof_elf_get_symbol_at_address_internal (self->debug_link_elf, filename, address, begin_address, end_address, text_offset);
 
       if (ret != NULL)
         return ret;
@@ -447,11 +446,7 @@ sysprof_elf_get_symbol_at_address_internal (SysprofElf *self,
     }
   else
     {
-      begin = address;
-      end = address + 1;
-      ret = g_strdup_printf ("In File %s+0x%"G_GINT64_MODIFIER"x", filename, address);
-      if (is_fallback)
-        *is_fallback = TRUE;
+      return NULL;
     }
 
   if (begin_address)
@@ -467,16 +462,14 @@ char *
 sysprof_elf_get_symbol_at_address (SysprofElf *self,
                                    guint64     address,
                                    guint64    *begin_address,
-                                   guint64    *end_address,
-                                   gboolean   *is_fallback)
+                                   guint64    *end_address)
 {
   return sysprof_elf_get_symbol_at_address_internal (self,
                                                      self->file,
                                                      address,
                                                      begin_address,
                                                      end_address,
-                                                     self->text_offset,
-                                                     is_fallback);
+                                                     self->text_offset);
 }
 
 /**
@@ -502,6 +495,7 @@ sysprof_elf_set_debug_link_elf (SysprofElf *self,
 {
   g_return_if_fail (SYSPROF_IS_ELF (self));
   g_return_if_fail (!debug_link_elf || SYSPROF_IS_ELF (debug_link_elf));
+  g_return_if_fail (debug_link_elf != self);
 
   if (g_set_object (&self->debug_link_elf, debug_link_elf))
     g_object_notify_by_pspec (G_OBJECT (self), properties [PROP_DEBUG_LINK_ELF]);
