@@ -30,6 +30,7 @@
 #include <glib/gi18n.h>
 #include <glib/gstdio.h>
 
+#include <libdex.h>
 #include <sysprof.h>
 
 #if HAVE_POLKIT_AGENT
@@ -38,6 +39,10 @@
 # include <polkitagent/polkitagent.h>
 #endif
 
+#if HAVE_LIBSYSTEMD
+# include <systemd/sd-daemon.h>
+#endif
+
 #include "sysprof-capture-util-private.h"
 
 #define DEFAULT_BUFFER_SIZE (1024L * 1024L * 8L /* 8mb */)
@@ -340,11 +345,12 @@ main (int   argc,
     { "monitor-bus", 0, 0, G_OPTION_ARG_STRING_ARRAY, &monitor_bus, N_("Additional D-Bus address to monitor") },
     { "stack-size", 0, 0, G_OPTION_ARG_INT, &stack_size, N_("Stack size to copy for unwinding in user-space") },
     { "no-debuginfod", 0, 0, G_OPTION_ARG_NONE, &disable_debuginfod, N_("Do not use debuginfod to resolve symbols") },
-    { "no-sysprofd", 0, 0, G_OPTION_ARG_NONE, &no_sysprofd, "Do not use Sysprofd to acquire privileges" },
+    { "no-sysprofd", 0, 0, G_OPTION_ARG_NONE, &no_sysprofd, N_("Do not use Sysprofd to acquire privileges") },
     { NULL }
   };
 
   sysprof_clock_init ();
+  dex_init ();
 
   g_unix_signal_add (SIGINT, sigint_handler, main_loop);
   g_unix_signal_add (SIGTERM, sigint_handler, main_loop);
@@ -625,6 +631,11 @@ Examples:\n\
 
   g_printerr ("Recording, press ^C to exit\n");
 
+#if HAVE_LIBSYSTEMD
+  if (command == NULL && child_argv == NULL)
+    sd_notify (TRUE, "READY=1");
+#endif
+
   g_main_loop_run (main_loop);
 
   sysprof_capture_writer_flush (writer);
