@@ -26,6 +26,8 @@
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
 
+#include <libdex.h>
+
 #include <sysprof-capture.h>
 
 #include "sysprof-application.h"
@@ -50,6 +52,11 @@ main (int argc,
 
   g_set_prgname ("sysprof");
 
+  dex_init ();
+
+  /* Setup thread pool scheduler at startup */
+  (void)dex_thread_pool_scheduler_get_default ();
+
   app = sysprof_application_new ();
   ret = g_application_run (G_APPLICATION (app), argc, argv);
 
