@@ -32,29 +32,36 @@ struct _SysprofApplication
   AdwApplication parent_instance;
 };
 
-G_DEFINE_TYPE (SysprofApplication, sysprof_application, ADW_TYPE_APPLICATION)
+G_DEFINE_FINAL_TYPE (SysprofApplication, sysprof_application, ADW_TYPE_APPLICATION)
 
 static const GOptionEntry option_entries[] = {
+  { "greeter", 'g', 0, G_OPTION_ARG_NONE, NULL, N_("Start a new recording") },
   { "version", 0, 0, G_OPTION_ARG_NONE, NULL, N_("Show Sysprof version and exit") },
   { 0 }
 };
 
+enum {
+  PROP_0,
+  PROP_DESCENDANT_MENU,
+  N_PROPS
+};
+
+static GParamSpec *properties[N_PROPS];
+
 static void
 sysprof_application_activate (GApplication *app)
 {
+  GtkWindow *window;
   GtkWidget *greeter;
 
   g_assert (GTK_IS_APPLICATION (app));
 
-  for (const GList *iter = gtk_application_get_windows (GTK_APPLICATION (app));
-       iter != NULL;
-       iter = iter->next)
+  window = gtk_application_get_active_window (GTK_APPLICATION (app));
+
+  if (window)
     {
-      if (SYSPROF_IS_WINDOW (iter->data))
-        {
-          gtk_window_present (iter->data);
-          return;
-        }
+      gtk_window_present (window);
+      return;
     }
 
   greeter = sysprof_greeter_new ();
@@ -79,6 +86,46 @@ sysprof_application_open (GApplication  *app,
     sysprof_window_open (SYSPROF_APPLICATION (app), template, files[i]);
 }
 
+static int
+sysprof_application_command_line (GApplication            *app,
+                                  GApplicationCommandLine *command_line)
+{
+  g_auto(GStrv) argv = NULL;
+  g_autoptr(GPtrArray) files = NULL;
+  GVariantDict *options;
+  int argc;
+  gboolean greeter = FALSE;
+
+  g_assert (SYSPROF_IS_APPLICATION (app));
+  g_assert (G_IS_APPLICATION_COMMAND_LINE (command_line));
+
+  argv = g_application_command_line_get_arguments (command_line, &argc);
+  options = g_application_command_line_get_options_dict (command_line);
+  files = g_ptr_array_new_with_free_func (g_object_unref);
+
+  for (int i = 1; i < argc; i++)
+    {
+      const char *filename = argv[i];
+      g_autoptr(GFile) file = NULL;
+      file = g_application_command_line_create_file_for_arg (command_line, filename);
+      g_ptr_array_add (files, g_steal_pointer (&file));
+    }
+
+  if (files->len > 0)
+    g_application_open (app,
+                        (GFile **)(gpointer)files->pdata,
+                        files->len,
+                        NULL);
+
+  if (g_variant_dict_lookup (options, "greeter", "b", &greeter) && greeter)
+    g_action_group_activate_action (G_ACTION_GROUP (app), "new-window", NULL);
+
+  if (files->len == 0 && !greeter)
+    g_application_activate (app);
+
+  return EXIT_SUCCESS;
+}
+
 static int
 sysprof_application_handle_local_options (GApplication *app,
                                           GVariantDict *options)
@@ -110,17 +157,77 @@ sysprof_application_window_added (GtkApplication *application,
   GTK_APPLICATION_CLASS (sysprof_application_parent_class)->window_added (application, window);
 }
 
+static void
+sysprof_application_get_property (GObject    *object,
+                                  guint       prop_id,
+                                  GValue     *value,
+                                  GParamSpec *pspec)
+{
+  SysprofApplication *self = SYSPROF_APPLICATION (object);
+
+  switch (prop_id)
+    {
+    case PROP_DESCENDANT_MENU:
+      g_value_set_object (value, gtk_application_get_menu_by_id (GTK_APPLICATION (self), "descendant_menu"));
+      break;
+
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+    }
+}
+
 static void
 sysprof_application_class_init (SysprofApplicationClass *klass)
 {
+  GObjectClass *object_class = G_OBJECT_CLASS (klass);
   GApplicationClass *app_class = G_APPLICATION_CLASS (klass);
   GtkApplicationClass *gtk_app_class = GTK_APPLICATION_CLASS (klass);
 
+  object_class->get_property = sysprof_application_get_property;
+
   app_class->open = sysprof_application_open;
   app_class->activate = sysprof_application_activate;
+  app_class->command_line = sysprof_application_command_line;
   app_class->handle_local_options = sysprof_application_handle_local_options;
 
   gtk_app_class->window_added = sysprof_application_window_added;
+
+  properties[PROP_DESCENDANT_MENU] =
+    g_param_spec_object ("descendant-menu", NULL, NULL,
+                         G_TYPE_MENU_MODEL,
+                         (G_PARAM_READABLE|
+                          G_PARAM_STATIC_STRINGS));
+
+  g_object_class_install_properties (object_class, N_PROPS, properties);
+}
+
+static void
+sysprof_new_window (GSimpleAction *action,
+                    GVariant      *variant,
+                    gpointer       user_data)
+{
+  const GList *windows;
+  GApplication *app = user_data;
+  GtkWidget *greeter;
+
+  g_assert (G_IS_APPLICATION (app));
+  g_assert (G_IS_SIMPLE_ACTION (action));
+  g_assert (variant == NULL);
+
+  windows = gtk_application_get_windows (GTK_APPLICATION (app));
+
+  for (const GList *iter = windows; iter != NULL; iter = iter->next)
+    {
+      if (SYSPROF_IS_GREETER (iter->data))
+        {
+          gtk_window_present (iter->data);
+          return;
+        }
+    }
+
+  greeter = sysprof_greeter_new ();
+  gtk_application_add_window (GTK_APPLICATION (app), GTK_WINDOW (greeter));
+  gtk_window_present (GTK_WINDOW (greeter));
 }
 
 static void
@@ -143,35 +250,24 @@ sysprof_about (GSimpleAction *action,
                gpointer       user_data)
 {
   GtkApplication *app = user_data;
-  GtkWindow *best_toplevel = NULL;
-  const GList *windows;
+  GtkWindow *window = NULL;
 
   g_assert (G_IS_APPLICATION (app));
   g_assert (G_IS_SIMPLE_ACTION (action));
   g_assert (variant == NULL);
 
-  windows = gtk_application_get_windows (app);
+  window = gtk_application_get_active_window (GTK_APPLICATION (app));
 
-  if (windows == NULL)
+  if (window == NULL)
     {
       g_warning ("No windows found to show about dialog");
-    }
-  else
-    best_toplevel = windows->data;
-
-  for (; windows != NULL; windows = windows->next)
-    {
-      if (SYSPROF_IS_WINDOW (windows->data))
-        {
-          best_toplevel = windows->data;
-          break;
-        }
+      return;
     }
 
-  adw_show_about_dialog (GTK_WIDGET (best_toplevel),
+  adw_show_about_dialog (GTK_WIDGET (window),
                          "application-name", _("Sysprof"),
                          "application-icon", APP_ID_S,
-                         "version", "GNOME " PACKAGE_VERSION,
+                         "version", PACKAGE_VERSION,
                          "copyright", "Copyright 2004-2009 Søren Sandmann Pedersen\n"
                                       "Copyright 2016-2023 Christian Hergert",
                          "issue-url", "https://gitlab.gnome.org/GNOME/sysprof/-/issues/new",
@@ -233,6 +329,7 @@ static void
 sysprof_application_init (SysprofApplication *self)
 {
   static const GActionEntry actions[] = {
+    { "new-window", sysprof_new_window },
     { "about", sysprof_about },
     { "show-help-overlay", sysprof_show_help_overlay },
     { "help",  sysprof_help },
@@ -264,6 +361,6 @@ sysprof_application_new (void)
   return g_object_new (SYSPROF_TYPE_APPLICATION,
                        "application-id", APP_ID_S,
                        "resource-base-path", "/org/gnome/sysprof",
-                       "flags", G_APPLICATION_HANDLES_OPEN,
+                       "flags", G_APPLICATION_HANDLES_COMMAND_LINE | G_APPLICATION_HANDLES_OPEN,
                        NULL);
 }
