@@ -42,22 +42,24 @@ struct _SysprofGreeter
   AdwWindow                 parent_instance;
 
   GtkStringList            *envvars;
+  GtkStringList            *debugdirs;
   SysprofRecordingTemplate *recording_template;
 
   AdwViewStack             *view_stack;
   GtkListBox               *sidebar_list_box;
   AdwPreferencesPage       *record_page;
   GtkListBox               *app_environment;
-  GtkSwitch                *sample_native_stacks;
-  GtkSwitch                *sample_javascript_stacks;
-  GtkSwitch                *record_disk_usage;
-  GtkSwitch                *record_network_usage;
-  GtkSwitch                *record_compositor;
-  GtkSwitch                *record_system_logs;
-  GtkSwitch                *record_session_bus;
-  GtkSwitch                *record_system_bus;
-  GtkSwitch                *bundle_symbols;
-  GtkSwitch                *debuginfod;
+  GtkListBox               *debug_directories;
+  AdwSwitchRow             *sample_native_stacks;
+  AdwSwitchRow             *sample_javascript_stacks;
+  AdwSwitchRow             *record_disk_usage;
+  AdwSwitchRow             *record_network_usage;
+  AdwSwitchRow             *record_compositor;
+  AdwSwitchRow             *record_system_logs;
+  AdwSwitchRow             *record_session_bus;
+  AdwSwitchRow             *record_system_bus;
+  AdwSwitchRow             *bundle_symbols;
+  AdwSwitchRow             *debuginfod;
   GtkButton                *record_to_memory;
   AdwComboRow              *power_combo;
   AdwComboRow              *sample_user_stack_size;
@@ -109,6 +111,20 @@ on_env_items_changed_cb (SysprofGreeter *self,
                           g_list_model_get_n_items (model) > 0);
 }
 
+static void
+on_debug_dir_items_changed_cb (SysprofGreeter *self,
+                               guint           position,
+                               guint           removed,
+                               guint           added,
+                               GListModel     *model)
+{
+  g_assert (SYSPROF_IS_GREETER (self));
+  g_assert (G_IS_LIST_MODEL (model));
+
+  gtk_widget_set_visible (GTK_WIDGET (self->debug_directories),
+                          g_list_model_get_n_items (model) > 0);
+}
+
 static void
 on_env_entry_changed_cb (SysprofGreeter      *self,
                          SysprofEntryPopover *popover)
@@ -205,6 +221,9 @@ sysprof_greeter_create_profiler (SysprofGreeter  *self,
   g_autoptr(GFile) dir = NULL;
   GtkStringObject *strobj;
   const char *str;
+  guint n_items;
+  g_autoptr(GStrvBuilder) builder = NULL;
+  g_auto(GStrv) debugdirs = NULL;
 
   g_assert (SYSPROF_IS_GREETER (self));
 
@@ -212,7 +231,7 @@ sysprof_greeter_create_profiler (SysprofGreeter  *self,
     {
       g_autoptr(GStrvBuilder) envvars_builder = NULL;
       g_auto(GStrv) envvars_list = NULL;
-      
+
       envvars_builder = g_strv_builder_new ();
 
       for (guint i = 0; i < g_list_model_get_n_items (G_LIST_MODEL (self->envvars)); i++)
@@ -232,6 +251,23 @@ sysprof_greeter_create_profiler (SysprofGreeter  *self,
                   "power-profile", str,
                   NULL);
 
+  if ((n_items = g_list_model_get_n_items (G_LIST_MODEL (self->debugdirs))))
+    {
+      builder = g_strv_builder_new ();
+
+      for (guint i = 0; i < n_items; i++)
+        {
+          strobj = g_list_model_get_item (G_LIST_MODEL (self->debugdirs), i);
+          g_strv_builder_add (builder, gtk_string_object_get_string (strobj));
+        }
+
+      debugdirs = g_strv_builder_end (builder);
+
+      g_object_set (self->recording_template,
+                    "debugdirs", debugdirs,
+                    NULL);
+    }
+
   if (!(profiler = sysprof_recording_template_apply (self->recording_template, error)))
     return NULL;
 
@@ -314,6 +350,45 @@ sysprof_greeter_choose_file_for_record_cb (GObject      *object,
                                          sysprof_greeter_record_cb,
                                          g_object_ref (self));
         }
+      else
+        {
+          AdwDialog *alert_dialog;
+
+          alert_dialog = adw_alert_dialog_new (_("Must Capture to Local File"), NULL);
+          adw_alert_dialog_format_body (ADW_ALERT_DIALOG (alert_dialog),
+                                        _("You must choose a local file to capture using Sysprof"));
+          adw_alert_dialog_add_responses (ADW_ALERT_DIALOG (alert_dialog),
+                                          "close", _("Close"),
+                                          NULL);
+          adw_alert_dialog_set_close_response (ADW_ALERT_DIALOG (alert_dialog), "cancel");
+          adw_alert_dialog_choose (ADW_ALERT_DIALOG (alert_dialog), GTK_WIDGET (self),
+                                   NULL, (GAsyncReadyCallback) sysprof_greeter_choose_file_for_record_cb, self);
+        }
+    }
+}
+
+static void
+sysprof_greeter_choose_folder_for_debug_dir (GObject      *object,
+                                             GAsyncResult *result,
+                                             gpointer      user_data)
+{
+  GtkFileDialog *dialog = (GtkFileDialog *)object;
+  g_autoptr(SysprofGreeter) self = user_data;
+  g_autoptr(GError) error = NULL;
+  g_autoptr(GFile) file = NULL;
+
+  g_assert (GTK_IS_FILE_DIALOG (dialog));
+  g_assert (G_IS_ASYNC_RESULT (result));
+  g_assert (SYSPROF_IS_GREETER (self));
+  g_assert (GTK_IS_STRING_LIST (self->debugdirs));
+
+  if ((file = gtk_file_dialog_select_folder_finish (dialog, result, &error)))
+    {
+      if (g_file_is_native (file))
+        {
+          const char *path = g_file_peek_path (file);
+          gtk_string_list_append (self->debugdirs, path);
+        }
       else
         {
           GtkWidget *message;
@@ -321,8 +396,8 @@ sysprof_greeter_choose_file_for_record_cb (GObject      *object,
           G_GNUC_BEGIN_IGNORE_DEPRECATIONS
 
           message = adw_message_dialog_new (NULL,
-                                            _("Must Capture to Local File"),
-                                            _("You must choose a local file to capture using Sysprof"));
+                                          _("Must Select Local Folder"),
+                                          _("You must choose a local folder to add as a debug directory"));
           adw_message_dialog_add_response (ADW_MESSAGE_DIALOG (message), "close", _("Close"));
           gtk_window_present (GTK_WINDOW (message));
 
@@ -331,6 +406,27 @@ sysprof_greeter_choose_file_for_record_cb (GObject      *object,
     }
 }
 
+static void
+sysprof_greeter_select_debug_directory (GtkWidget  *widget,
+                                        const char *action_name,
+                                        GVariant   *param)
+{
+  SysprofGreeter *self = (SysprofGreeter *)widget;
+  g_autoptr(GtkFileDialog) dialog = NULL;
+
+  g_assert (SYSPROF_IS_GREETER (self));
+
+  dialog = gtk_file_dialog_new ();
+  gtk_file_dialog_set_title (dialog, _("Add debug directory"));
+  gtk_file_dialog_set_accept_label (dialog, _("Add _Directory"));
+  gtk_file_dialog_set_modal (dialog, TRUE);
+  gtk_file_dialog_select_folder (dialog,
+                                 GTK_WINDOW (self),
+                                 NULL,
+                                 sysprof_greeter_choose_folder_for_debug_dir,
+                                 g_object_ref (self));
+}
+
 static void
 sysprof_greeter_record_to_file_action (GtkWidget  *widget,
                                        const char *action_name,
@@ -453,6 +549,31 @@ delete_envvar_cb (SysprofGreeter *self,
     }
 }
 
+static void
+delete_debugdirs_cb (SysprofGreeter *self,
+                     GtkButton      *button)
+{
+  const char *debug_directory;
+  guint n_items;
+
+  g_assert (SYSPROF_IS_GREETER (self));
+  g_assert (GTK_IS_BUTTON (button));
+
+  debug_directory = g_object_get_data (G_OBJECT (button), "DEBUGDIR");
+  n_items = g_list_model_get_n_items (G_LIST_MODEL (self->debugdirs));
+
+  for (guint i = 0; i < n_items; i++)
+    {
+      g_autoptr(GtkStringObject) str = g_list_model_get_item (G_LIST_MODEL (self->debugdirs), i);
+
+      if (g_strcmp0 (debug_directory, gtk_string_object_get_string (str)) == 0)
+        {
+          gtk_string_list_remove (self->debugdirs, i);
+          break;
+        }
+    }
+}
+
 static GtkWidget *
 create_envvar_row_cb (gpointer item,
                       gpointer user_data)
@@ -493,6 +614,46 @@ create_envvar_row_cb (gpointer item,
 
   return GTK_WIDGET (row);
 }
+static GtkWidget *
+create_debugdirs_row_cb (gpointer item,
+                         gpointer user_data)
+{
+  SysprofGreeter *self = user_data;
+  GtkStringObject *obj = item;
+  const char *str;
+  g_autofree char *markup = NULL;
+  g_autofree char *escaped = NULL;
+  AdwActionRow *row;
+  GtkButton *button;
+
+  g_assert (SYSPROF_IS_GREETER (self));
+  g_assert (GTK_IS_STRING_OBJECT (obj));
+
+  str = gtk_string_object_get_string (obj);
+  escaped = g_markup_escape_text (str, -1);
+  markup = g_strdup_printf ("<tt>%s</tt>", escaped);
+  row = g_object_new (ADW_TYPE_ACTION_ROW,
+                      "title", markup,
+                      "title-selectable", TRUE,
+                      NULL);
+  button = g_object_new (GTK_TYPE_BUTTON,
+                         "icon-name", "list-remove-symbolic",
+                         "css-classes", STRV_INIT ("flat", "circular"),
+                         "valign", GTK_ALIGN_CENTER,
+                         NULL);
+  g_object_set_data_full (G_OBJECT (button),
+                          "DEBUGDIR",
+                          g_strdup (str),
+                          g_free);
+  g_signal_connect_object (button,
+                           "clicked",
+                           G_CALLBACK (delete_debugdirs_cb),
+                           self,
+                           G_CONNECT_SWAPPED);
+  adw_action_row_add_suffix (row, GTK_WIDGET (button));
+
+  return GTK_WIDGET (row);
+}
 
 static char *
 translate_power_profile (GtkStringObject *strobj)
@@ -542,6 +703,7 @@ sysprof_greeter_dispose (GObject *object)
   gtk_widget_dispose_template (GTK_WIDGET (self), SYSPROF_TYPE_GREETER);
 
   g_clear_object (&self->envvars);
+  g_clear_object (&self->debugdirs);
   g_clear_object (&self->recording_template);
 
   G_OBJECT_CLASS (sysprof_greeter_parent_class)->dispose (object);
@@ -558,9 +720,11 @@ sysprof_greeter_class_init (SysprofGreeterClass *klass)
   gtk_widget_class_set_template_from_resource (widget_class, "/org/gnome/sysprof/sysprof-greeter.ui");
 
   gtk_widget_class_bind_template_child (widget_class, SysprofGreeter, app_environment);
+  gtk_widget_class_bind_template_child (widget_class, SysprofGreeter, debug_directories);
   gtk_widget_class_bind_template_child (widget_class, SysprofGreeter, bundle_symbols);
   gtk_widget_class_bind_template_child (widget_class, SysprofGreeter, debuginfod);
   gtk_widget_class_bind_template_child (widget_class, SysprofGreeter, envvars);
+  gtk_widget_class_bind_template_child (widget_class, SysprofGreeter, debugdirs);
   gtk_widget_class_bind_template_child (widget_class, SysprofGreeter, power_combo);
   gtk_widget_class_bind_template_child (widget_class, SysprofGreeter, record_compositor);
   gtk_widget_class_bind_template_child (widget_class, SysprofGreeter, record_disk_usage);
@@ -584,6 +748,7 @@ sysprof_greeter_class_init (SysprofGreeterClass *klass)
   gtk_widget_class_bind_template_callback (widget_class, on_env_entry_changed_cb);
   gtk_widget_class_bind_template_callback (widget_class, translate_power_profile);
 
+  gtk_widget_class_install_action (widget_class, "system.select-debug-directory", NULL, sysprof_greeter_select_debug_directory);
   gtk_widget_class_install_action (widget_class, "win.record-to-memory", NULL, sysprof_greeter_record_to_memory_action);
   gtk_widget_class_install_action (widget_class, "win.record-to-file", NULL, sysprof_greeter_record_to_file_action);
   gtk_widget_class_install_action (widget_class, "win.select-file", NULL, sysprof_greeter_select_file_action);
@@ -612,8 +777,17 @@ sysprof_greeter_init (SysprofGreeter *self)
                            G_CALLBACK (on_env_items_changed_cb),
                            self,
                            G_CONNECT_SWAPPED);
+
   on_env_items_changed_cb (self, 0, 0, 0, G_LIST_MODEL (self->envvars));
 
+  g_signal_connect_object (self->debugdirs,
+                           "items-changed",
+                           G_CALLBACK (on_debug_dir_items_changed_cb),
+                           self,
+                           G_CONNECT_SWAPPED);
+
+  on_debug_dir_items_changed_cb (self, 0, 0, 0, G_LIST_MODEL (self->debugdirs));
+
   gtk_list_box_bind_model (self->sidebar_list_box,
                            G_LIST_MODEL (adw_view_stack_get_pages (self->view_stack)),
                            sysprof_greeter_create_sidebar_row,
@@ -624,16 +798,34 @@ sysprof_greeter_init (SysprofGreeter *self)
                            create_envvar_row_cb,
                            self, NULL);
 
+   gtk_list_box_bind_model (self->debug_directories,
+                            G_LIST_MODEL (self->debugdirs),
+                            create_debugdirs_row_cb,
+                            self, NULL);
+
   if (self->recording_template)
     {
-      g_auto(GStrv) environ = NULL;
+      g_auto(GStrv) environ_ = NULL;
       g_object_get (self->recording_template,
-                    "environ", &environ,
+                    "environ", &environ_,
                     NULL);
-      if (environ)
+      if (environ_)
         {
-          for (guint i = 0; environ[i]; i++)
-            gtk_string_list_append (self->envvars, environ[i]);            
+          for (guint i = 0; environ_[i]; i++)
+            gtk_string_list_append (self->envvars, environ_[i]);
+        }
+    }
+
+  if (self->recording_template)
+    {
+      g_auto(GStrv) debug_directory_list = NULL;
+      g_object_get (self->recording_template,
+                    "debugdirs", &debug_directory_list,
+                    NULL);
+      if (debug_directory_list)
+        {
+          for (guint i = 0; debug_directory_list[i]; i++)
+            gtk_string_list_append (self->debugdirs, debug_directory_list[i]);
         }
     }
 
