@@ -41,6 +41,7 @@
 #include "sysprof-pair.h"
 #include "sysprof-processes-section.h"
 #include "sysprof-samples-section.h"
+#include "sysprof-session-filters-widget.h"
 #include "sysprof-sidebar.h"
 #include "sysprof-storage-section.h"
 #include "sysprof-task-row.h"
@@ -112,8 +113,6 @@ sysprof_window_update_action_state (SysprofWindow *self)
 
   if (self->session == NULL)
     {
-      gtk_widget_action_set_enabled (GTK_WIDGET (self), "win.open-capture", FALSE);
-      gtk_widget_action_set_enabled (GTK_WIDGET (self), "win.record-capture", FALSE);
       gtk_widget_action_set_enabled (GTK_WIDGET (self), "session.zoom-one", FALSE);
       gtk_widget_action_set_enabled (GTK_WIDGET (self), "session.zoom-out", FALSE);
       gtk_widget_action_set_enabled (GTK_WIDGET (self), "session.zoom-in", FALSE);
@@ -123,8 +122,6 @@ sysprof_window_update_action_state (SysprofWindow *self)
     }
   else
     {
-      gtk_widget_action_set_enabled (GTK_WIDGET (self), "win.open-capture", TRUE);
-      gtk_widget_action_set_enabled (GTK_WIDGET (self), "win.record-capture", TRUE);
       gtk_widget_action_set_enabled (GTK_WIDGET (self), "win.save-capture", TRUE);
       gtk_widget_action_set_enabled (GTK_WIDGET (self), "session.zoom-in", TRUE);
 
@@ -227,24 +224,6 @@ sysprof_window_open_capture_action (GtkWidget  *widget,
   sysprof_window_open_file (GTK_WINDOW (widget), template);
 }
 
-static void
-sysprof_window_record_capture_action (GtkWidget  *widget,
-                                      const char *action_name,
-                                      GVariant   *param)
-{
-  SysprofWindow *self = (SysprofWindow *)widget;
-  SysprofGreeter *greeter;
-
-  g_assert (SYSPROF_IS_WINDOW (self));
-
-  greeter = g_object_new (SYSPROF_TYPE_GREETER,
-                          "transient-for", self,
-                          NULL);
-  gtk_application_add_window (GTK_APPLICATION (SYSPROF_APPLICATION_DEFAULT),
-                              GTK_WINDOW (greeter));
-  gtk_window_present (GTK_WINDOW (greeter));
-}
-
 static void
 sysprof_window_set_document (SysprofWindow   *self,
                              SysprofDocument *document)
@@ -328,6 +307,13 @@ main_view_notify_sidebar (SysprofWindow       *self,
   gtk_widget_set_sensitive (GTK_WIDGET (self->show_right_sidebar), sidebar != NULL);
 }
 
+static gboolean
+n_filters_to_button_visibility (SysprofWindow *self,
+                                unsigned int   n_filters)
+{
+  return n_filters > 0;
+}
+
 static void
 sysprof_window_session_seek_backward (GtkWidget  *widget,
                                       const char *action_name,
@@ -617,9 +603,9 @@ sysprof_window_class_init (SysprofWindowClass *klass)
   gtk_widget_class_bind_template_child (widget_class, SysprofWindow, stack_title);
 
   gtk_widget_class_bind_template_callback (widget_class, main_view_notify_sidebar);
+  gtk_widget_class_bind_template_callback (widget_class, n_filters_to_button_visibility);
 
   gtk_widget_class_install_action (widget_class, "win.open-capture", NULL, sysprof_window_open_capture_action);
-  gtk_widget_class_install_action (widget_class, "win.record-capture", NULL, sysprof_window_record_capture_action);
   gtk_widget_class_install_action (widget_class, "session.zoom-one", NULL, sysprof_window_session_zoom_one);
   gtk_widget_class_install_action (widget_class, "session.zoom-out", NULL, sysprof_window_session_zoom_out);
   gtk_widget_class_install_action (widget_class, "session.zoom-in", NULL, sysprof_window_session_zoom_in);
@@ -649,6 +635,7 @@ sysprof_window_class_init (SysprofWindowClass *klass)
   g_type_ensure (SYSPROF_TYPE_NETWORK_SECTION);
   g_type_ensure (SYSPROF_TYPE_PROCESSES_SECTION);
   g_type_ensure (SYSPROF_TYPE_SAMPLES_SECTION);
+  g_type_ensure (SYSPROF_TYPE_SESSION_FILTERS_WIDGET);
   g_type_ensure (SYSPROF_TYPE_STORAGE_SECTION);
   g_type_ensure (SYSPROF_TYPE_SESSION);
   g_type_ensure (SYSPROF_TYPE_SYMBOL);
